@@ -363,6 +363,14 @@ public interface UserManager {
 	 * @throws UserRetrievalFault
 	 */
 	List<GCubeUser> listUsersByGroupAndRole(long groupId, long roleId) throws UserManagementSystemException, RoleRetrievalFault, GroupRetrievalFault, UserRetrievalFault;
+	/**
+	 * 
+	 * @param groupId
+	 * @param roleId
+	 * @return the list of userId having group and role that matches
+	 */
+	long[] listUserIdsByGroupAndRole(long groupId, long roleId);
+	/**
 	/**
 	 * 
 	 * @parmam teamId the LR teamId
