@@ -25,6 +25,7 @@ import org.gcube.vomanagement.usermanagement.model.GCubeGroup;
 import org.gcube.vomanagement.usermanagement.model.GCubeMembershipRequest;
 import org.gcube.vomanagement.usermanagement.model.GCubeRole;
 import org.gcube.vomanagement.usermanagement.model.GCubeUser;
+import org.gcube.vomanagement.usermanagement.model.GatewayRolesNames;
 import org.gcube.vomanagement.usermanagement.model.MembershipRequestStatus;
 import org.gcube.vomanagement.usermanagement.util.ManagementUtils;
 
@@ -45,6 +46,7 @@ import com.liferay.portal.model.Role;
 import com.liferay.portal.model.RoleConstants;
 import com.liferay.portal.model.Team;
 import com.liferay.portal.model.User;
+import com.liferay.portal.model.UserGroupRole;
 import com.liferay.portal.security.auth.PrincipalThreadLocal;
 import com.liferay.portal.security.permission.PermissionChecker;
 import com.liferay.portal.security.permission.PermissionCheckerFactoryUtil;
@@ -57,6 +59,7 @@ import com.liferay.portal.service.RoleLocalServiceUtil;
 import com.liferay.portal.service.RoleServiceUtil;
 import com.liferay.portal.service.ServiceContext;
 import com.liferay.portal.service.TeamLocalServiceUtil;
+import com.liferay.portal.service.UserGroupRoleLocalServiceUtil;
 import com.liferay.portal.service.UserLocalServiceUtil;
 import com.liferay.portal.util.PortalUtil;
 import com.liferay.portal.webserver.WebServerServletTokenUtil;
@@ -94,6 +97,31 @@ public class LiferayUserManager implements UserManager {
 		}
 		return null;
 	}
+	/**
+	 * 
+	 * @param contextGroupId
+	 * @param gatewayRole
+	 * @return the User if a user having the role exists in the Site group (Gateway, VO or VRE)
+	 */
+	public static User getRandomUserWithRole(long contextGroupId, GatewayRolesNames gatewayRole) {
+		final long companyId = PortalUtil.getDefaultCompanyId();
+		try {
+			Role theRole = RoleLocalServiceUtil.getRole(companyId, gatewayRole.getRoleName()) ;
+			if (theRole == null) {
+				_log.warn("No user has the role with this groupId " + gatewayRole.getRoleName() + ":" + contextGroupId);
+				return null;
+			}
+			List<UserGroupRole> list = UserGroupRoleLocalServiceUtil.getUserGroupRolesByGroupAndRole(contextGroupId, theRole.getRoleId());
+			for (UserGroupRole userGroupRole : list) {
+				User admin = userGroupRole.getUser();
+				if (admin.isActive())
+					return admin;
+			}
+		} catch (final Exception e) {
+			_log.error("Utils::getInfrastructureManager Exception", e);
+		}
+		return null;
+	}
 	public static Role getRoleById(final long companyId, final String roleStrId) {
 		try {
 			return RoleLocalServiceUtil.getRole(companyId, roleStrId);
@@ -753,24 +781,34 @@ public class LiferayUserManager implements UserManager {
 	 */
 	@Override
 	public List<GCubeUser> listUsersByGroupAndRole(long groupId, long roleId) throws UserManagementSystemException, RoleRetrievalFault,	GroupRetrievalFault, UserRetrievalFault {
-		Map<GCubeUser, List<GCubeRole>> toIterate = listUsersAndRolesByGroup(groupId);
 		List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
-		Role askedRole = null;
 		try {
-			askedRole = RoleServiceUtil.getRole(roleId);
-		} catch (PortalException e) {
-			throw new RoleRetrievalFault("Role not existing (I think you better check) roleId="+roleId, e);
+			List<UserGroupRole> listTriplet = UserGroupRoleLocalServiceUtil.getUserGroupRolesByGroupAndRole(groupId, roleId);
+			for (UserGroupRole userGroupRole : listTriplet) {
+				toReturn.add(getUserById(userGroupRole.getUserId()));
+			}
+		} catch (SystemException e1) {
+			e1.printStackTrace();
+		}	
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public long[] listUserIdsByGroupAndRole(long groupId, long roleId) {
+		List<UserGroupRole> listTriplet = null;
+		try {
+			listTriplet = UserGroupRoleLocalServiceUtil.getUserGroupRolesByGroupAndRole(groupId, roleId);
 		} catch (SystemException e) {
 			e.printStackTrace();
 		}
-		_log.debug("Asked for role: "+askedRole.getName());	
-		for (GCubeUser u : toIterate.keySet()) {
-			for (GCubeRole role : toIterate.get(u)) 
-				if (role.getRoleId() == roleId) {
-					toReturn.add(u);
-					break;
-				}				
-		}		
+		long[] toReturn = new long[listTriplet.size()];
+		int i = 0;
+		for (UserGroupRole userGroupRole : listTriplet) {
+			toReturn[i] = userGroupRole.getUserId();
+			i++;
+		}
 		return toReturn;
 	}
 	/**
